
/* /////////////////////////////////////////////////
                    ArduiRotor  
   /////////////////////////////////////////////////

   Commande d'un rotor (0/359°) par DTMF
   Utilise un décodeur DTMF MT8870
   Version 1.0
   
   Codes DTMF  = "D" suivi d'un nombre de 3 chiffres entre 000 et 359.  
   Acquittement OK ou NOK par envoi d'un signal graphie ("OK" ou "HI").

   Un code de blocage, ici  D 7 8 *   et un code de déblocage  D 8 7 * sont présents.
   Ces codes peuvent être modifiés au sein du programme.

   Un contrôle du système est  possible via le moniteur série Arduino.

   F5MNA Janvier 2016  http://f5mna.free.fr

   /////////////////////////////////////
*/

int DTMF;
int ConfDTMF = 3;     // Confirmation de réception d'un code DTMF Pin 3
int DTMF0 = 10;       // Sortie 0 MT8870 Pin  4
int DTMF1 = 11;       // Sortie 1 MT8870      5
int DTMF2 = 12;       // Sortie 2 MT8870      6
int DTMF3 = 13;       // Sortie 3 MT8870      7

int Led = 2;          // Led
int pin0 = 0;         // analog pin Mesuure potentiomètre de recopie du rotor.

int Buffer[15];    // Mémorisation du code
int Cod;        // Nombre de codes à transmettre
int Code;          // Codes à transmettre
int val;           // confirmation DTMF (ConfDTMF)
int buttonState;   // Prise en compte du changement d'état de la confirmation DTMF
boolean OK = false;// Gestion codes
boolean Blocage = false;// Gestion codes
boolean Marche = false;
int Tempo;         // Fenetre de temporisation.
int degres, Tension, TensionDeg, Acqui;

///CW
int PTT = 9;          // PTT TX
int pinBF = 5;
int freq = 800;  // Fréquence
int Di = 100;
int Da = 150;

// Marche Droite/gauche
int AV = 6;
int AR = 7;

void setup()
{
  Serial.begin (9600);
  pinMode (ConfDTMF, INPUT); // Validation code DTMF
  pinMode (DTMF0, INPUT); // Sortie MT8870
  pinMode (DTMF1, INPUT); // Sortie MT8870
  pinMode (DTMF2, INPUT); // Sortie MT8870
  pinMode (DTMF3, INPUT); // Sortie MT8870
  pinMode (Led, OUTPUT);     // Led
  pinMode (PTT, OUTPUT);     // PTT TX
  pinMode (pinBF, OUTPUT);   // Notes CW
  pinMode (AV, OUTPUT);     // PTT TX
  pinMode (AR, OUTPUT);   // Notes CW

  Cod = 0;
  DTMF = 0;        // Sommes sorties DTMF
  Tempo = 0;
  buttonState = digitalRead(ConfDTMF);
  
  Serial.println ("       Test ArduiRotor");
  Serial.println ();

}

void Acquit ()
{
  if (Acqui == 1) {
    Serial.println ("  Acquittement OK");
    digitalWrite(PTT, HIGH);  // TX en marche
    delay (500);
    tone(pinBF, freq, Di);
    delay (Da);
    tone(pinBF, freq, Di);
    delay (Da);  
    digitalWrite(PTT, LOW);  // TX Arrêt
  }
  if (Acqui == 0) {
    Serial.println ("  Acquittement NOK");
    digitalWrite(PTT, HIGH);  // TX en marche
    delay (500);
    tone(pinBF, freq, Di);
    delay (Da);
    tone(pinBF, freq, Di);
    delay (Da);
    tone(pinBF, freq, Di);
    delay (Da);
    tone(pinBF, freq, Di);
    delay (Da * 2);
    tone(pinBF, freq, Di);
    delay (Da);
    tone(pinBF, freq, Di);
    delay (Da * 6);
    digitalWrite(PTT, LOW);  // TX Arrêt
  }
}

void Rotor()
{
  Tension =  analogRead(pin0);
  TensionDeg = (2.84 * degres); // (360/1023)* degres
  if (Tension > TensionDeg) {
    Serial.println ("Gauche");
    digitalWrite (AR, HIGH);
    digitalWrite (AV, LOW);
  }
  if (Tension < TensionDeg) {
    Serial.println ("Droite");
    digitalWrite (AR, LOW);
    digitalWrite (AV, HIGH);
  }
  if (Tension == TensionDeg) {
    Serial.print("S T O P  ");
    Serial.print (degres);
    Serial.print (" degres");
    digitalWrite (AR, LOW);
    digitalWrite (AV, LOW);
    delay (200);
    Marche = false;
    Acqui = 1;
    Acquit();
  }
}


void Commande()
{
  //////////////////////////////////////////
  Serial.print ("Valeur en degres:  ");
  if (Buffer[1] == 10) Buffer[1] = 0;
  if (Buffer[2] == 10) Buffer[2] = 0;
  if (Buffer[3] == 10) Buffer[3] = 0;

  int Centaine = (Buffer[1]) * 100;
  int dizaine  = Buffer[2] * 10;
  degres = Centaine + dizaine + Buffer[3];
  Serial.print (degres);
  Serial.println ();
  Serial.println ();
  Cod = 0;

  if ((degres < 360) and (Blocage == false)) {
    Marche = true;
    Acqui = 1;
    Acquit ();
    Rotor();
  }
  else
  { Serial.println ("   Erreur: Valeur non compatible");
    Acqui = 0;
    Acquit ();
  }
}

void loop ()  {
  val = digitalRead(ConfDTMF); //Confirmation DTMF
  if (val != buttonState) { // Modification de cette valeur
    if (val == LOW)
    {
      DTMF = 0;
      Tempo = 0;
      if (digitalRead(DTMF3) == HIGH) DTMF = DTMF + 1;
      if (digitalRead(DTMF2) == HIGH) DTMF = DTMF + 2;
      if (digitalRead(DTMF1) == HIGH) DTMF = DTMF + 4;
      if (digitalRead(DTMF0) == HIGH) DTMF = DTMF + 8;
      switch (DTMF)
      {
        case 0: // "D"
          OK = true; ///////////////////////////////////
          Buffer[Cod] = 0;
          break;
        case 1: // "1"
          Buffer[Cod] = 1;
          break;
        case 2: // "2"
          Buffer[Cod] = 2;
          break;
        case 3: // "3"
          Buffer[Cod] = 3;
          break;
        case 4: // "4"
          Buffer[Cod] = 4;
          break;
        case 5: // "5"
          Buffer[Cod] = 5;
          break;
        case 6: // "6"
          Buffer[Cod] = 6;
          break;
        case 7: // "7"
          Buffer[Cod] = 7;
          break;
        case 8: // "8"
          Buffer[Cod] = 8;
          break;
        case 9:// "9"
          Buffer[Cod] = 9;
          break;
        case 10: // "0"
          Buffer[Cod] = 10;
          break;
        case 11: // "*"
          Buffer[Cod] = 11;
          break;
        case 12: // "#"
          Buffer[Cod] = 12;  
          Serial.print("S T O P  ");
          digitalWrite (AR, LOW);
          digitalWrite (AV, LOW);
          Serial.println ("S T O P  ");
          Serial.print (Tension / 2.84);
          Serial.println (" degres");
          delay (200);
    Marche = false;
    Acqui = 1;
    Acquit();          
          // Remise à zero des codes ////////////////////////////
          break;
        case 13: // "A"
          Buffer[Cod] = 13;
          break;
        case 14: // "B"
          Buffer[Cod] = 14;
          break;
        case 15: // "C"
          Buffer[Cod] = 15;
          break;
      }

      digitalWrite(Led, HIGH);
      delay (200);
      digitalWrite(Led, LOW);
      Serial.print ("Entree code    ");
      Serial.print (DTMF);
      Serial.print ("     ");
      Serial.print (Cod);
      Serial.print ("     ");
      Serial.println (OK);

      //Prise en compte de 4 codes
      ////////////////////////////
      if (OK == true) (Cod ++);

      if (Cod > 3) {
        Tempo == 0;
        Cod == 0;
        OK = false;
        Commande();
      }
    }
  }
  buttonState = val; // Confirmation de la validation DTMF (CM8870)(confirmation changement).

  // Fenetre de temporisation
  if (OK == true) {
    Tempo ++;
    delay (5);
  }
  if ((Cod < 5) && (Tempo > 1000)) {
    Serial.println ("Hors temporisation");
    Tempo = 0;
    Cod = 0;
    OK = false;  
  }
  if (Marche == true) Rotor();

  /////////////////////////////////////////////////////////////
  ///Blocage  Deblocage du système  ( D 7 8 *) et (D 8 7 *) ///
  /////////////////////////////////////////////////////////////

  if ((Buffer[1] == 7) && (Buffer[2] == 8) && (Buffer[3] == 11))
  { Serial.println ("Blocage");
    Blocage = true;
    digitalWrite(PTT, LOW);
  }

  if ((Buffer[1] == 8) && (Buffer[2] == 7) && (Buffer[3] == 11))
  { Blocage = false;
    Serial.println ("Deblocage");
  }
}
// FIN



